# -*- coding: windows-1252 -*-
test_inbound_payload_1252 = {}
test_inbound_payload_1252['text'] = """This is a message in windows-1252 encoding. åßœ∑´¥¨¨^"""
test_inbound_payload_1252['charsets'] = '{"to":"UTF-8","html":"utf-8","subject":"UTF-8","from":"UTF-8","text":"windows-1252"}'
test_inbound_payload_1252['subject'] = 'test'
test_inbound_payload_1252['to'] = '<to@example.com>'
test_inbound_payload_1252['cc'] = '<b@example.com>, <c@example.com>'
test_inbound_payload_1252['from'] = 'Hugo Rodger-Brown <from@example.com>'



# test contents taken from the Mailgun route tester - https://mailgun.com/cp/routes#
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

test_inbound_payload = {}
test_inbound_payload['stripped-signature']="""Thanks,
Bob"""
test_inbound_payload['From']='Bob <bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['attachment-count']='2'
test_inbound_payload['To']='Alice <alice@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['subject']='Re: Sample POST request'
test_inbound_payload['from']='Bob <bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['User-Agent']='Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/20130308 Thunderbird/17.0.4'
test_inbound_payload['stripped-html']="""<html><head><meta content="text/html; charset=ISO-8859-1" http-equiv="Content-Type"></head><body text="#000000" bgcolor="#FFFFFF">
    <div class="moz-cite-prefix">
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Hi Alice,</div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);"><br></div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">This is Bob.<span class="Apple-converted-space">&#160;<img alt="" src="cid:part1.04060802.06030207@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org" height="15" width="33"></span></div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);"><br>
        I also attached a file.<br><br></div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Thanks,</div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Bob</div>
      <br><br></div>
    <br></body></html>"""
test_inbound_payload['In-Reply-To']='<517AC78B.5060404@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['Date']='Fri, 26 Apr 2013 11:50:29 -0700'
test_inbound_payload['Message-Id']='<517ACC75.5010709@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['body-plain']="""Hi Alice,

This is Bob.

I also attached a file.

Thanks,
Bob

On 04/26/2013 11:29 AM, Alice wrote:
> Hi Bob,
>
> This is Alice. How are you doing?
>
> Thanks,
> Alice

"""
test_inbound_payload['Mime-Version']='1.0'
test_inbound_payload['Received']='from [10.20.76.69] (Unknown [50.56.129.169]) by mxa.mailgun.org with ESMTP id 517acc75.4b341f0-worker2; Fri, 26 Apr 2013 18:50:29 -0000 (UTC)'
test_inbound_payload['content-id-map']='{"<part1.04060802.06030207@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>": "attachment-1"}'
test_inbound_payload['Sender']='bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org'
test_inbound_payload['timestamp']='1412239366'
test_inbound_payload['message-headers']='[["Received", "by luna.mailgun.net with SMTP mgrt 8788212249833; Fri, 26 Apr 2013 18:50:30 +0000"], ["Received", "from [10.20.76.69] (Unknown [50.56.129.169]) by mxa.mailgun.org with ESMTP id 517acc75.4b341f0-worker2; Fri, 26 Apr 2013 18:50:29 -0000 (UTC)"], ["Message-Id", "<517ACC75.5010709@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>"], ["Date", "Fri, 26 Apr 2013 11:50:29 -0700"], ["From", "Bob <bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>"], ["User-Agent", "Mozilla/5.0 (X11; Linux x86_64; rv:17.0) Gecko/20130308 Thunderbird/17.0.4"], ["Mime-Version", "1.0"], ["To", "Alice <alice@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>"], ["Subject", "Re: Sample POST request"], ["References", "<517AC78B.5060404@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>"], ["In-Reply-To", "<517AC78B.5060404@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>"], ["X-Mailgun-Variables", "{\"my_var_1\": \"Mailgun Variable #1\", \"my-var-2\": \"awesome\"}"], ["Content-Type", "multipart/mixed; boundary=\"------------020601070403020003080006\""], ["Sender", "bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org"]]'
test_inbound_payload['stripped-text']="""Hi Alice,

This is Bob.

I also attached a file."""
test_inbound_payload['recipient']='alice@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org'
test_inbound_payload['sender']='bob@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org'
test_inbound_payload['X-Mailgun-Variables']='{"my_var_1": "Mailgun Variable #1", "my-var-2": "awesome"}'
test_inbound_payload['token']='4166ec05b27b5d3c024252ad43df47865128c8fcde47dfea6b'
test_inbound_payload['body-html']="""<html>
  <head>
    <meta content="text/html; charset=ISO-8859-1"
      http-equiv="Content-Type">
  </head>
  <body text="#000000" bgcolor="#FFFFFF">
    <div class="moz-cite-prefix">
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Hi Alice,</div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);"><br>
      </div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">This is Bob.<span class="Apple-converted-space">&nbsp;<img
            alt="" src="cid:part1.04060802.06030207@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org"
            height="15" width="33"></span></div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);"><br>
        I also attached a file.<br>
        <br>
      </div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Thanks,</div>
      <div style="color: rgb(34, 34, 34); font-family: arial,
        sans-serif; font-size: 12.666666984558105px; font-style: normal;
        font-variant: normal; font-weight: normal; letter-spacing:
        normal; line-height: normal; orphans: auto; text-align: start;
        text-indent: 0px; text-transform: none; white-space: normal;
        widows: auto; word-spacing: 0px; -webkit-text-size-adjust: auto;
        -webkit-text-stroke-width: 0px; background-color: rgb(255, 255,
        255);">Bob</div>
      <br>
      On 04/26/2013 11:29 AM, Alice wrote:<br>
    </div>
    <blockquote cite="mid:517AC78B.5060404@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org" type="cite">Hi
      Bob,
      <br>
      <br>
      This is Alice. How are you doin©?
      <br>
      <br>
      Thanks,
      <br>
      Alice
      <br>
    </blockquote>
    <br>
  </body>
</html>
"""
test_inbound_payload['References']='<517AC78B.5060404@sandbox085a3b878a964897b9b9efb2395a1f5a.mailgun.org>'
test_inbound_payload['signature']='b220bb34b816a1b00512e1a288ac9b8e5808c51de773bc13628547486eb3a80d'
test_inbound_payload['Content-Type']='multipart/mixed; boundary="------------020601070403020003080006"'
test_inbound_payload['Subject']='Re: Sample POST request'



# -*- coding: utf-8 -*-
"""Python 2/3 compatibility imports."""
try:
    from unittest import mock
except ImportError:
    import mock  # noqa

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse  # noqa



# -*- coding: utf-8 -*-
from django.conf.urls import url
from . import views

urlpatterns = [
    url(
        r'^inbound/$',
        views.receive_inbound_email,
        name='receive_inbound_email'
    )
]



# encoding=utf-8
from __future__ import unicode_literals

import re
import hashlib
import hmac
import json
import logging
import base64

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.http import HttpRequest
from django.utils.encoding import smart_bytes

from ..backends import RequestParser
from ..errors import (
    RequestParseError,
    AttachmentTooLargeError,
    AuthenticationError,
)

logger = logging.getLogger(__name__)


class MandrillSignatureMismatchError(AuthenticationError):
    """Error raised when the request's mandrill signature doesn't match.
    """

    def __init__(self, request, expected, calculated):
        super(MandrillSignatureMismatchError, self)
        self.request = request
        self.expected_signature = expected
        self.calculated_signature = calculated


def _detect_base64(s):
    """Quite an ingenuous function to guess if a string is base64 encoded
    """
    return (len(s) % 4 == 0) and re.match('^[A-Za-z0-9+/]+[=]{0,2}$', s)


def _check_mandrill_signature(request, key):
    expected = request.META.get('HTTP_X_MANDRILL_SIGNATURE', None)
    url = request.build_absolute_uri()
    # Mandrill appends the POST params in alphabetical order of the key.
    params = sorted(request.POST.items(), key=lambda x: x[0])
    message = url + ''.join(key + value for key, value in params)
    signed_binary = hmac.new(
        key.encode('utf-8'),
        message.encode('utf-8'),
        hashlib.sha1,
    )
    signature = base64.b64encode(signed_binary.digest()).decode('utf-8')
    if signature != expected:
        raise MandrillSignatureMismatchError(request, expected, signature)


class MandrillRequestParser(RequestParser):
    """Mandrill request parser. """

    def _process_attachments(self, email, attachments):
        for key, attachment in list(attachments.items()):
            is_base64 = attachment.get('base64')
            name = attachment.get('name')
            mimetype = attachment.get('type')
            content = attachment.get('content', "")

            if is_base64:
                content = base64.b64decode(content)
            # watchout: sometimes attachment contents are base64'd but mandrill doesn't set the flag
            elif _detect_base64(content):
                content = base64.b64decode(content)

            content = smart_bytes(content, strings_only=True)

            if len(content) > self.max_file_size:
                logger.debug(
                    "File attachment %s is too large to process (%sB)",
                    name,
                    len(content)
                )
                raise AttachmentTooLargeError(
                    email=email,
                    filename=name,
                    size=len(content)
                )

            if name and mimetype and content:
                email.attach(name, content, mimetype)
        return email

    def _get_recipients(self, array):
        """Returns an iterator of objects
           in the form ["Name <address@example.com", ...]
           from the array [["address@example.com", "Name"]]
        """
        for address, name in array:
            if not name:
                yield address
            else:
                yield "\"%s\" <%s>" % (name, address)

    def _get_sender(self, from_email, from_name=None):
        if not from_name:
            return from_email
        else:
            return "\"%s\" <%s>" % (from_name, from_email)

    def parse(self, request):
        """Parse incoming request and return an email instance.

        Args:
            request: an HttpRequest object, containing a list of forwarded emails, as
                per Mandrill specification for inbound emails.

        Returns:
            a list of EmailMultiAlternatives instances
        """
        assert isinstance(request, HttpRequest), "Invalid request type: %s" % type(request)

        if settings.INBOUND_MANDRILL_AUTHENTICATION_KEY:
            _check_mandrill_signature(
                request=request,
                key=settings.INBOUND_MANDRILL_AUTHENTICATION_KEY,
            )

        try:
            messages = json.loads(request.POST['mandrill_events'])
        except (ValueError, KeyError) as ex:
            raise RequestParseError("Request is not a valid json: %s" % ex)

        if not messages:
            logger.debug("No messages found in mandrill request: %s", request.body)
            return []

        emails = []
        for message in messages:
            if message.get('event') != 'inbound':
                logger.debug("Discarding non-inbound message")
                continue

            msg = message.get('msg')
            try:
                from_email = msg['from_email']
                to = list(self._get_recipients(msg['to']))

                subject = msg.get('subject', "")

                attachments = msg.get('attachments', {})
                attachments.update(msg.get('images', {}))

                text = msg.get('text', "")
                html = msg.get('html', "")
            except (KeyError, ValueError) as ex:
                raise RequestParseError(
                    "Inbound request is missing or got an invalid value.: %s." % ex
                )

            email = EmailMultiAlternatives(
                subject=subject,
                body=text,
                from_email=self._get_sender(
                    from_email=from_email,
                    from_name=msg.get('from_name'),
                ),
                to=to,
            )
            if html is not None and len(html) > 0:
                email.attach_alternative(html, "text/html")

            email = self._process_attachments(email, attachments)
            emails.append(email)

        return emails



test_inbound_payload = {}
test_inbound_payload['text'] = """This is a message.


Sent with Unibox



"""
test_inbound_payload['charsets'] = '{"to":"UTF-8","html":"utf-8","subject":"UTF-8","from":"UTF-8","text":"utf-8"}'
test_inbound_payload['subject'] = 'test'
test_inbound_payload['to'] = '<to@example.com>'
test_inbound_payload['cc'] = '<b@example.com>, <c@example.com>'
test_inbound_payload['spam_score'] = '0.266'
test_inbound_payload['html'] = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"><head><title></title></head><body><div>This is a message.
</div><div><br></div><div class="unibox-signature">Sent with <a href="http://www.uniboxapp.com/t/sig">Unibox</a>
</div></body></html>"""
test_inbound_payload['from'] = 'Hugo Rodger-Brown <from@example.com>'
test_inbound_payload['sender_ip'] = '74.125.82.43'
test_inbound_payload['attachment-info'] = '{"attachment1":{"filename":"Screen Shot 2014-05-12 at 17.06.14.png","name":"Screen Shot 2014-05-12 at 17.06.14.png","type":"image/png"}}'
test_inbound_payload['envelope'] = '{"to":["to@example.com"],"from":"from@example.com"}'
test_inbound_payload['dkim'] = 'none'
test_inbound_payload['SPF'] = 'pass'
test_inbound_payload['headers'] = """Received: by mx-007.sjc1.sendgrid.net with SMTP id og7B28feuJ Tue, 13 May 2014 10:25:40 +0000 (GMT)
Received: from mail-wg0-f43.google.com (mail-wg0-f43.google.com [74.125.82.43]) by mx-007.sjc1.sendgrid.net (Postfix) with ESMTPS id 5FE7FA836D6 for <A1@messages-uat.yunojuno.com>; Tue, 13 May 2014 10:25:39 +0000 (GMT)
Received: by mail-wg0-f43.google.com with SMTP id l18so147281wgh.14 for <to@example.com>; Tue, 13 May 2014 03:25:37 -0700 (PDT)
X-Google-DKIM-Signature: v=1; a=rsa-sha256; c=relaxed/relaxed; d=1e100.net; s=20130820; h=x-gm-message-state:from:to:subject:date:message-id:mime-version :content-type; bh=9o+Uxa2TnusbKHc7VtEFg22821Db2LqhnYwxDDCfcKY=; b=LSth5QmBnqBJEgrshQlZ3mBoxut8fiQqoLbk+qdEpi/ZhEcKPTe0zePlBsEGqhAgUK 4HedAUcRqY8okISA+I3Rma9pP0y2lrqpedSrRi8M6y1ywXOyeDY1eVGCO9VqyXk1q4Fo c7xUXkQbRst9CP03BrDTLHcGwzdlbmV1fk/WhJqp2N0DVnYfSaBVF/MRovPdLvLxmuz7 Sbb//SDySNCue3AiyhJ/NEhyrI4CsTJo54fPdhBUkW8GMNurrKoBZVlnDRmRNTXgN8Jz vu8k0yhFeGKCMv5BuosFLaCQAGwzvu6G7NfIfMBLeIloEB5VJrq7ZclzY9EwQOllEbjv i3yA==
X-Gm-Message-State: ALoCoQlleb+6UgRbdYDnkbup4PQhdsCZlpK/KEPBFlKr4jabqxRWQSKNMsfv/lHMq1NTG8sTqB0m
X-Received: by 10.194.87.163 with SMTP id az3mr1760074wjb.63.1399976737521; Tue, 13 May 2014 03:25:37 -0700 (PDT)
Received: from [192.168.0.106] ([217.138.17.50]) by mx.google.com with ESMTPSA id fz11sm21154279wic.4.2014.05.13.03.25.35 for <to@example.com> (version=TLSv1 cipher=ECDHE-RSA-RC4-SHA bits=128/128); Tue, 13 May 2014 03:25:35 -0700 (PDT)
X-Google-Original-Date: 13 May 2014 11:25:36 +0100
From: Hugo Rodger-Brown <from@example.com>
To: <to@example.com>
Subject: test
Date: Tue, 13 May 2014 03:25:35 -0700 (PDT)
Message-Id: <E31C8C77-1EE4-41C9-A503-2C028F224D7C@example.com>
MIME-Version: 1.0
X-Mailer: Unibox (188)
Content-Type: multipart/mixed; boundary="=_C7FE9D5A-F282-4723-97C8-88C72CAE2F38"
"""
test_inbound_payload['spam_report'] = """Spam detection software, running on the system "mx-007.sjc1.sendgrid.net", has
identified this incoming email as possible spam.  The original message
has been attached to this so you can view it (if it isn't spam) or label
similar future email.  If you have any questions, see
the administrator of that system for details.

Content preview:  This is a message. Sent with Unibox This is a message. [...]


Content analysis details:   (0.3 points, 5.0 required)

 pts rule name              description
---- ---------------------- --------------------------------------------------
 0.0 HTML_MESSAGE           BODY: HTML included in message
 0.0 DC_PNG_UNO_LARGO       Message contains a single large inline gif
 0.1 DC_IMAGE_SPAM_TEXT     Possible Image-only spam with little text
 0.1 DC_IMAGE_SPAM_HTML     Possible Image-only spam

"""
test_inbound_payload['attachments'] = '1'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from os import path

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.test import TestCase, override_settings
from django.test.client import RequestFactory

from ..errors import (
    AttachmentTooLargeError,
    AuthenticationError,
)
from ..signals import email_received, email_received_unacceptable
from ..views import receive_inbound_email, _log_request

from .test_files.sendgrid_post import test_inbound_payload as sendgrid_payload
from .test_files.mailgun_post import test_inbound_payload as mailgun_payload
from .test_files.mandrill_post import post_data as mandrill_payload
from .test_files.mandrill_post import (
    post_data_with_attachments as mandrill_payload_with_attachments
)


# don't read it out of the settings - fix it here so we know what we're using
DEFAULT_TEST_PARSER = "inbound_email.backends.sendgrid.SendGridRequestParser"
MANDRILL_REQUEST_PARSER = "inbound_email.backends.mandrill.MandrillRequestParser"
SENDGRID_REQUEST_PARSER = "inbound_email.backends.sendgrid.SendGridRequestParser"
MAILGUN_REQUEST_PARSER = "inbound_email.backends.mailgun.MailgunRequestParser"


class ViewFunctionTests(TestCase):
    """Tests for the inbound view function receive_inbound_email.

    The view function is responsible for loading the correct backend, and
    firing the signal once the email is parsed. This test suite contains no
    parsing tests - these are covered in the relevant backend tests - just tests
    for the signals.

    """
    def setUp(self):
        # Every test needs access to the request factory.
        self.factory = RequestFactory()
        self.url = reverse('receive_inbound_email')
        self.test_upload_txt = path.join(path.dirname(__file__), 'test_files/test_upload_file.txt')

    def _get_payloads_and_parsers(self, with_attachments=False):
        mpwa = mandrill_payload_with_attachments
        mp = mandrill_payload
        return [
            (MANDRILL_REQUEST_PARSER, mpwa if with_attachments else mp),
            (SENDGRID_REQUEST_PARSER, sendgrid_payload),
            (MAILGUN_REQUEST_PARSER, mailgun_payload),
        ]

    def test_log_inbound_requests(self):
        """Test the internal log function."""
        # just to exercise the function - it doesn't 'do' anything other than
        # log to the console, but is good to know that it doesn't break.

        for klass, payload in self._get_payloads_and_parsers():
            settings.INBOUND_EMAIL_PARSER = klass
            request = self.factory.post(self.url, data=payload)
            _log_request(request)

    def test_inbound_request_HEAD_200(self):
        """Return 200 OK to a HEAD request."""
        request = self.factory.head(self.url)
        response = receive_inbound_email(request)
        self.assertEqual(response.status_code, 200)

    def test_valid_request(self):
        """Test the RequestParseErrors are handled correctly, and return HTTP 200."""
        for klass, payload in self._get_payloads_and_parsers():
            settings.INBOUND_EMAIL_PARSER = klass
            request = self.factory.post(self.url, data=payload)
            response = receive_inbound_email(request)
            self.assertContains(response, "Successfully parsed", status_code=200)

    def test_parse_error_response_200(self):
        """Test the RequestParseErrors are handled correctly, and return HTTP 200."""
        settings.INBOUND_EMAIL_RESPONSE_200 = True
        for klass, payload in self._get_payloads_and_parsers():
            settings.INBOUND_EMAIL_PARSER = klass
            request = self.factory.post(self.url, data={})
            response = receive_inbound_email(request)
            self.assertContains(response, "Unable to parse", status_code=200)

    def test_parse_error_response_400(self):
        """Test the RequestParseErrors are handled correctly, and return HTTP 400."""
        settings.INBOUND_EMAIL_RESPONSE_200 = False
        request = self.factory.post(self.url, data={})
        response = receive_inbound_email(request)
        self.assertContains(response, "Unable to parse", status_code=400)

    def test_email_received_signal(self):
        """Test that a valid POST fires the email_received signal."""
        # define handler
        for klass, payload in self._get_payloads_and_parsers():

            def on_email_received(sender, **kwargs):
                self.on_email_received_fired = True
                self.assertEqual(sender.__name__, klass.split('.')[-1])
                request = kwargs.pop('request', None)
                email = kwargs.pop('email', None)
                self.assertIsNotNone(email)
                self.assertIsInstance(email, EmailMultiAlternatives)
                self.assertIsNotNone(request)
            email_received.connect(on_email_received)

            settings.INBOUND_EMAIL_PARSER = klass
            request = self.factory.post(self.url, data=payload)

            # connect handler
            self.on_email_received_fired = False

            # fire a request in to force the signal to fire
            receive_inbound_email(request)
            self.assertTrue(self.on_email_received_fired)

            email_received.disconnect(on_email_received)

    def test_email_received_unacceptable_signal_fired_for_too_large_attachment(self):
        # set a zero allowed max attachment size
        settings.INBOUND_EMAIL_ATTACHMENT_SIZE_MAX = 0

        for klass, payload in self._get_payloads_and_parsers(with_attachments=True):
            settings.INBOUND_EMAIL_PARSER = klass
            _payload = payload.copy()

            if klass == SENDGRID_REQUEST_PARSER:
                _payload['attachment'] = open(self.test_upload_txt, 'r')
            if klass == MAILGUN_REQUEST_PARSER:
                _payload['attachment-1'] = open(self.test_upload_txt, 'r')

            # define handler
            def on_email_received(sender, **kwargs):
                self.on_email_received_fired = True
                request = kwargs.pop('request', None)
                email = kwargs.pop('email', None)
                exception = kwargs.pop('exception', None)
                self.assertEqual(sender.__name__, klass.split('.')[-1])
                self.assertIsNotNone(request)
                self.assertIsInstance(email, EmailMultiAlternatives)
                self.assertIsInstance(exception, AttachmentTooLargeError)
            email_received_unacceptable.connect(on_email_received)

            self.on_email_received_fired = False

            request = self.factory.post(self.url, data=_payload)
            receive_inbound_email(request)
            self.assertTrue(self.on_email_received_fired, klass)
            email_received_unacceptable.disconnect(on_email_received)

    @override_settings(INBOUND_MANDRILL_AUTHENTICATION_KEY='mandrill_key')
    def test_email_received_unacceptable_signal_fired_for_mandrill_mistmatch_signature(self):
        parser = MANDRILL_REQUEST_PARSER
        payload = mandrill_payload
        settings.INBOUND_EMAIL_PARSER = parser
        _payload = payload.copy()

        # define handler
        def on_email_received(sender, **kwargs):
            self.on_email_received_fired = True
            request = kwargs.pop('request', None)
            email = kwargs.pop('email', None)
            exception = kwargs.pop('exception', None)
            self.assertEqual(sender.__name__, parser.split('.')[-1])
            self.assertIsNotNone(request)
            self.assertIsNone(email,)
            self.assertIsInstance(exception, AuthenticationError)
        email_received_unacceptable.connect(on_email_received)

        self.on_email_received_fired = False

        request = self.factory.post(
            self.url,
            data=_payload,
            HTTP_X_MANDRILL_SIGNATURE='invalid_signature',
        )
        receive_inbound_email(request)
        self.assertTrue(self.on_email_received_fired, parser)
        email_received_unacceptable.disconnect(on_email_received)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import hashlib
import hmac
import json
import base64

from django.test import TestCase, override_settings
from django.test.client import RequestFactory
from django.utils.encoding import smart_bytes

from ..backends.mandrill import (
    MandrillRequestParser,
    MandrillSignatureMismatchError,
)
from ..compat import reverse
from ..errors import RequestParseError, AttachmentTooLargeError

from .test_files.mandrill_post import post_data as mandrill_payload
from .test_files.mandrill_post import (
    post_data_with_attachments as mandrill_payload_with_attachments
)
from .test_files.mandrill_post import (
    post_data_with_attachments_mailbox as mandrill_payload_with_attachments_mailbox
)
from .test_files.mandrill_post import (
    post_data_with_attachments_mailbox_2 as mandrill_payload_with_attachments_mailbox_2
)


class MandrillRequestParserTests(TestCase):
    def setUp(self):
        self.url = reverse('receive_inbound_email')
        self.factory = RequestFactory()
        self.parser = MandrillRequestParser()
        self.payload = mandrill_payload
        self.payload_with_attachments = mandrill_payload_with_attachments

    def _assertEmailParsedCorrectly(self, emails, mandrill_payload, has_html=True):
        def _parse_to(to):
            ret = []
            for address, name in to:
                if not name:
                    ret.append(address)
                else:
                    ret.append("\"%s\" <%s>" % (name, address))
            return ret

        def _parse_from(name, email):
            if not name:
                return email
            return "\"%s\" <%s>" % (name, email)

        for i, e in enumerate(emails):
            msg = json.loads(mandrill_payload['mandrill_events'])[i]['msg']
            self.assertEqual(e.subject, msg['subject'])
            self.assertEqual(e.to, _parse_to(msg['to']))
            self.assertEqual(
                e.from_email,
                _parse_from(msg.get('from_name'), msg.get('from_email'))
            )
            if has_html:
                self.assertEqual(e.alternatives[0][0], msg['html'])
            for name, contents, mimetype in e.attachments:
                # Check that base64 contents are decoded
                is_base64 = msg['attachments'][name].get('base64')
                req_contents = msg['attachments'][name]['content']
                if is_base64:
                    req_contents = base64.b64decode(req_contents)
                self.assertEqual(smart_bytes(req_contents), smart_bytes(contents))
                self.assertEqual(msg['attachments'][name]['type'], mimetype)
            self.assertEqual(e.body, msg['text'])

    def _calculate_signature(self, url, data, key):
        # Mandrill appends the POST params in alphabetical order of the key.
        params = sorted(data, key=lambda x: x[0])
        message = url + ''.join(key + value for key, value in params)
        signed_binary = hmac.new(
            key.encode('utf-8'),
            message.encode('utf-8'),
            hashlib.sha1,
        )
        signature = base64.b64encode(signed_binary.digest())
        return signature.decode('utf-8')

    def test_parse_valid_request__with_attachments__from_mailbox(self):
        "Test email sent via Mailbox app without body text "
        request = self.factory.post(self.url, data=mandrill_payload_with_attachments_mailbox)
        emails = self.parser.parse(request)
        email = emails[0]
        payload = json.loads(mandrill_payload_with_attachments_mailbox['mandrill_events'])

        self.assertEqual(len(email.attachments), 1)
        self.assertEqual(email.attachments[0][0], '3c8e4ffb-6366-4351-813f-d0f600ed720e')
        self.assertEqual(email.attachments[0][2], 'image/jpeg')
        self.assertEqual(
            email.attachments[0][1],
            # the content is base64-decoded, even if the base64 flag is off in the request
            base64.b64decode(
                payload[0]['msg']['images']
                ['3c8e4ffb-6366-4351-813f-d0f600ed720e']['content']
            )
        )

    def test_parse_valid_request__with_attachments__from_mailbox__2(self):
        "Test email sent via Mailbox app with text "
        request = self.factory.post(self.url, data=mandrill_payload_with_attachments_mailbox_2)
        emails = self.parser.parse(request)
        email = emails[0]
        payload = json.loads(mandrill_payload_with_attachments_mailbox_2['mandrill_events'])

        self.assertEqual(len(email.attachments), 1)
        self.assertEqual(email.attachments[0][0], '7e357447-3f2e-4c12-a643-5720f30ca7af')
        self.assertEqual(email.attachments[0][2], 'image/jpeg')
        self.assertEqual(
            email.attachments[0][1],
            # the content is base64-decoded, even if the base64 flag is off in the request
            base64.b64decode(
                payload[0]['msg']['images']
                ['7e357447-3f2e-4c12-a643-5720f30ca7af']['content']
            )
        )

    def test_parse_valid_request(self):
        """
        Test a valid POST returns 200 and email(s) are correctly parsed
        """
        request = self.factory.post(self.url, data=mandrill_payload)
        emails = self.parser.parse(request)
        self._assertEmailParsedCorrectly(emails, mandrill_payload)

    def test_parse_valid_request__with_attachments(self):
        request = self.factory.post(self.url, data=mandrill_payload_with_attachments)
        emails = self.parser.parse(request)
        self._assertEmailParsedCorrectly(emails, mandrill_payload_with_attachments)

    @override_settings(INBOUND_MANDRILL_AUTHENTICATION_KEY='mandrill_key')
    def test_parse_valid_request__with_signature(self):
        signature = self._calculate_signature(
            url='http://testserver' + self.url,
            data=mandrill_payload.items(),
            key='mandrill_key',
        )
        request = self.factory.post(
            self.url,
            data=mandrill_payload,
            HTTP_X_MANDRILL_SIGNATURE=signature,
        )
        emails = self.parser.parse(request)
        self._assertEmailParsedCorrectly(emails, mandrill_payload)

    @override_settings(INBOUND_MANDRILL_AUTHENTICATION_KEY='mandrill_key')
    def test_parse_valid_request__with_invalid_signature(self):
        signature = 'invalid_signature'
        request = self.factory.post(
            self.url,
            data=mandrill_payload,
            HTTP_X_MANDRILL_SIGNATURE=signature,
        )
        self.assertRaises(
            MandrillSignatureMismatchError,
            self.parser.parse,
            request,
        )

    def _process_dump(self, foo):
        dump = json.loads(self.payload['mandrill_events'])
        foo(dump)
        return json.dumps(dump)

    def test_parse_invalid_request(self):
        """Test that an invalid request raises RequestParseError."""
        _process = lambda dump: dump[0]['msg'].pop('from_email', None)

        payload = {'mandrill_events': self._process_dump(_process)}
        request = self.factory.post(self.url, data=payload)
        self.assertRaises(RequestParseError, self.parser.parse, request)

    def test_text_email_only(self):
        """Test inbound email with no HTML alternative."""
        _process = lambda dump: dump[0]['msg'].pop('html')

        payload = {'mandrill_events': self._process_dump(_process)}
        request = self.factory.post(self.url, data=payload)
        email = self.parser.parse(request)
        self._assertEmailParsedCorrectly(email, payload, has_html=False)

    @override_settings(INBOUND_EMAIL_ATTACHMENT_SIZE_MAX=0)
    def test_attachments_max_size(self):
        """Test inbound email attachment max size limit."""
        # receive an email
        request = self.factory.post(self.url, data=self.payload_with_attachments)
        # should except
        with self.assertRaises(AttachmentTooLargeError):
            self.parser.parse(request)

    def test_correspondent_field_parsing(self):
        """Test the speific address parsing of the Mandrill backend"""
        # Addresses https://github.com/yunojuno/django-inbound-email/issues/20

        for input_data, attr_name, expected in [
            # TO / CC / BCC -- all handled by the same method: _get_recipients()
            (
                [['jed@whitehouse.gov', "Jed Bartlet"]],
                "to",
                ['"Jed Bartlet" <jed@whitehouse.gov>'],
            ),
            (
                [
                    ['jed@whitehouse.gov', "Jed Bartlet"],
                    ['toby@whitehouse.gov', "Toby Ziegler"]],
                "to",
                [
                    '"Jed Bartlet" <jed@whitehouse.gov>',
                    '"Toby Ziegler" <toby@whitehouse.gov>'
                ],
            ),
            (
                [['jed@whitehouse.gov', "Bartlet, Jed"]],
                "to",
                ['"Bartlet, Jed" <jed@whitehouse.gov>'],
            ),
            (
                [
                    ['jed@whitehouse.gov', "Bartlet, Jed"],
                    ['toby@whitehouse.gov', "Ziegler, Toby"]],
                "to",
                [
                    '"Bartlet, Jed" <jed@whitehouse.gov>',
                    '"Ziegler, Toby" <toby@whitehouse.gov>'
                ],
            ),
            (
                [['jed@whitehouse.gov', None]],
                "to",
                ['jed@whitehouse.gov'],
            ),
            (
                [
                    ['jed@whitehouse.gov', None],
                    ['toby@whitehouse.gov', None]],
                "to",
                [
                    'jed@whitehouse.gov',
                    'toby@whitehouse.gov'
                ],
            ),
            (
                [['jed@whitehouse.gov', '']],
                "to",
                ['jed@whitehouse.gov'],
            ),
            (
                [
                    ['jed@whitehouse.gov', ''],
                    ['toby@whitehouse.gov', '']
                ],
                "to",
                [
                    'jed@whitehouse.gov',
                    'toby@whitehouse.gov'
                ],
            ),

            # FROM - handled by a _get_sender() method
            # NB: 1) we do not get back a list.
            # 2) _get_sender expects email and name as args
            (
                ['jed@whitehouse.gov', "Jed Bartlet"],
                "from_email",
                '"Jed Bartlet" <jed@whitehouse.gov>',
            ),
            (
                ['jed@whitehouse.gov', "Bartlet, Jed"],
                "from_email",
                '"Bartlet, Jed" <jed@whitehouse.gov>',
            ),
            (
                ['jed@whitehouse.gov', ""],
                "from_email",
                'jed@whitehouse.gov',
            ),
            (
                ['jed@whitehouse.gov', None],
                "from_email",
                'jed@whitehouse.gov',
            ),

            # Handling edge-case or invalid content
            (
                # Latin 1 example
                [['sīla@exañple.com', "McTøst, Sīla"]],
                "to",
                ['"McTøst, Sīla" <sīla@exañple.com>'],
            ),
        ]:

            if attr_name == 'from_email':
                # _get_sender expects a two-tuple of args and returns a string
                output = self.parser._get_sender(*input_data)
            else:
                # _get_recipients expects a list of lists and returns a list of strings
                _results = self.parser._get_recipients(input_data)
                output = [x for x in _results]

            self.assertEqual(
                output,
                expected,
                "Failed to get %s for '%s' %s -- got %s" % (
                    expected, attr_name, input_data, output
                )
            )



from os import path, pardir, chdir
from setuptools import setup, find_packages

README = open(path.join(path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
chdir(path.normpath(path.join(path.abspath(__file__), pardir)))

setup(
    name='django-inbound-email',
    version='0.11.2',
    packages=find_packages(),
    install_requires=['Django>=1.8'],
    include_package_data=True,
    description='A Django app for receiving inbound emails.',
    license='MIT',
    long_description=README,
    url='https://github.com/yunojuno/django-inbound-email',
    author='YunoJuno',
    author_email='code@yunojuno.com',
    maintainer='YunoJuno',
    maintainer_email='code@yunojuno.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from os import path

from django.core.mail import EmailMultiAlternatives
from django.test import TestCase, override_settings
from django.test.client import RequestFactory
from django.utils.encoding import smart_bytes

from ..backends.mailgun import MailgunRequestParser
from ..compat import reverse
from ..errors import RequestParseError, AttachmentTooLargeError

from .test_files.mailgun_post import test_inbound_payload as mailgun_payload


class MailgunRequestParserTests(TestCase):
    """Tests for MailRequestParser - NB test use parser direct, not via view."""

    def _assertEmailParsedCorrectly(self, email, data):
        """Helper assert method that matches email properties to posted data.

        It's pulled out here as this is used in most of the tests to assert that
        the parser worked properly.

        Args:
            email: the EmailMultiAlternatives object to test.
            data: a dict of data that was sent to the parser.

        """
        self.assertIsInstance(email, EmailMultiAlternatives)
        self.assertEqual(email.to, data.get('recipient', '').split(','))
        self.assertEqual(email.from_email, data.get('sender', ''))
        self.assertEqual(email.subject, data.get('subject', ''))
        self.assertEqual(email.body, "%s\n\n%s" % (
            data.get('stripped-text', ''),
            data.get('stripped-signature', '')
        ))
        self.assertEqual(email.cc, data.get('cc', '').split(','))
        self.assertEqual(email.bcc, data.get('bcc', '').split(','))
        if 'html' in data:
            self.assertEqual(len(email.alternatives), 1)
            self.assertEqual(email.alternatives[0][0], data.get('stripped-html', ''))

    def setUp(self):
        # Every test needs access to the request factory.
        self.url = reverse('receive_inbound_email')
        self.factory = RequestFactory()
        self.parser = MailgunRequestParser()
        self.test_upload_txt = path.join(path.dirname(__file__), 'test_files/test_upload_file.txt')
        self.test_upload_png = path.join(path.dirname(__file__), 'test_files/test_upload_file.jpg')

    def test_parse_valid_request(self):
        """Test that a valid POST returns a 200."""
        request = self.factory.post(self.url, data=mailgun_payload)
        email = self.parser.parse(request)
        self._assertEmailParsedCorrectly(email, mailgun_payload)

    def test_parse_invalid_request(self):
        """Test that an invalid request raises RequestParseError."""
        request = self.factory.post(self.url, data={})
        self.assertRaises(RequestParseError, self.parser.parse, request)

    def test_text_email_only(self):
        """Test inbound email with no HTML alternative."""
        data = mailgun_payload
        del data['stripped-html']
        request = self.factory.post(self.url, data=data)
        email = self.parser.parse(request)
        self._assertEmailParsedCorrectly(email, mailgun_payload)

    def test_attachments(self):
        """Test inbound email with attachments."""
        data = mailgun_payload
        attachment_1 = open(self.test_upload_txt, 'r').read()
        attachment_2 = open(self.test_upload_png, 'rb').read()
        data['attachment-1'] = open(self.test_upload_txt, 'r')
        data['attachment-2'] = open(self.test_upload_png, 'rb')
        request = self.factory.post(self.url, data=data)
        email = self.parser.parse(request)

        self._assertEmailParsedCorrectly(email, data)

        # for each attachmen, check the contents match the input
        self.assertEqual(len(email.attachments), 2)

        # convert list of 3-tuples into dict so we can lookup by filename
        attachments = {k[0]: (k[1], k[2]) for k in email.attachments}
        self.assertEqual(smart_bytes(attachments['attachment-1'][0]), smart_bytes(attachment_1))
        self.assertEqual(attachments['attachment-1'][1], 'text/plain')
        self.assertEqual(attachments['attachment-2'][0], attachment_2)
        self.assertEqual(attachments['attachment-2'][1], 'image/jpeg')

    @override_settings(INBOUND_EMAIL_ATTACHMENT_SIZE_MAX=0)
    def test_attachments_max_size(self):
        """Test inbound email attachment max size limit."""
        # receive an email
        data = mailgun_payload
        open(self.test_upload_txt, 'rb').read()
        open(self.test_upload_png, 'rb').read()
        data['attachment-1'] = open(self.test_upload_txt, 'rb')
        data['attachment-2'] = open(self.test_upload_png, 'rb')
        request = self.factory.post(self.url, data=data)

        # should except
        with self.assertRaises(AttachmentTooLargeError):
            self.parser.parse(request),
